"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkDeviceConfig = void 0;
const logging_1 = require("./logging");
class NetworkDeviceConfig {
    constructor(config) {
        // WARNING!
        // This code might be dangerous, if you're going alone, please take this:
        //       /| _________________
        // O|===|* >________________/
        //       \|
        // Jokes aside, this code is really nasty. If you know of a better way
        // of solving it, please consider contributing:
        // https://github.com/AlexGustafsson/homebridge-wol#contribute
        this.name = "My NetworkDevice";
        this.manufacturer = "homebridge-wol";
        this.model = "NetworkDevice";
        this.host = null;
        this.pingInterval = 2;
        this.pingsToChange = 5;
        this.pingTimeout = 1;
        this.pingCommand = null;
        this.pingCommandTimeout = 0;
        this.mac = null;
        this.broadcastAddress = null;
        this.startCommand = null;
        this.startCommandTimeout = 0;
        this.wakeGraceTime = 45;
        this.wakeCommand = null;
        this.wakeCommandTimeout = 0;
        this.shutdownCommand = null;
        this.shutdownGraceTime = 15;
        this.shutdownCommandTimeout = 0;
        this.logLevel = logging_1.NetworkDeviceLogLevel.Info;
        this.returnEarly = false;
        this.name = this.getString(config, "name", this.name);
        this.manufacturer = this.getString(config, "manufacturer", this.manufacturer);
        this.model = this.getString(config, "model", this.model);
        this.serialNumber = this.getString(config, "serialNumber", new Array(4)
            .fill(null)
            .map((_) => Math.round(Math.random() * 1e5).toString())
            .join("-"));
        this.host = this.getString(config, "host", this.host);
        this.pingInterval =
            this.getNumber(config, "pingInterval", this.pingInterval) * 1000;
        this.pingsToChange = this.getNumber(config, "pingsToChange", this.pingsToChange);
        this.pingTimeout =
            this.getNumber(config, "pingTimeout", this.pingTimeout) * 1000;
        this.pingCommand = this.getString(config, "pingCommand", this.pingCommand);
        this.pingCommandTimeout =
            this.getNumber(config, "pingCommandTimeout", this.pingCommandTimeout) *
                1000;
        this.mac = this.getString(config, "mac", this.mac);
        this.broadcastAddress = this.getString(config, "broadcastAddress", this.broadcastAddress);
        this.startCommand = this.getString(config, "startCommand", this.startCommand);
        this.startCommandTimeout =
            this.getNumber(config, "startCommandTimeout", this.startCommandTimeout) *
                1000;
        this.wakeGraceTime =
            this.getNumber(config, "wakeGraceTime", this.wakeGraceTime) * 1000;
        this.wakeCommand = this.getString(config, "wakeCommand", this.wakeCommand);
        this.wakeCommandTimeout =
            this.getNumber(config, "wakeCommandTimeout", this.wakeCommandTimeout) *
                1000;
        this.shutdownCommand = this.getString(config, "shutdownCommand", this.shutdownCommand);
        this.shutdownGraceTime =
            this.getNumber(config, "shutdownGraceTime", this.shutdownGraceTime) *
                1000;
        this.shutdownCommandTimeout =
            this.getNumber(config, "shutdownCommandTimeout", this.shutdownCommandTimeout) * 1000;
        this.logLevel = this.getLogLevel(config, "logLevel", this.logLevel);
        this.returnEarly = this.getBoolean(config, "returnEarly", this.returnEarly);
    }
    getString(config, key, defaultValue) {
        if (typeof config[key] === "undefined")
            return defaultValue;
        const type = typeof config[key];
        if (type !== "string")
            throw new Error(`Got incorrect value type for config key '${key}' expected string, got '${type}'`);
        return config[key];
    }
    getNumber(config, key, defaultValue) {
        if (typeof config[key] === "undefined")
            return defaultValue;
        const type = typeof config[key];
        if (type !== "number")
            throw new Error(`Got incorrect value type for config key '${key}' expected number, got '${type}'`);
        return config[key];
    }
    getBoolean(config, key, defaultValue) {
        if (typeof config[key] === "undefined")
            return defaultValue;
        const type = typeof config[key];
        if (type !== "boolean")
            throw new Error(`Got incorrect value type for config key '${key}' expected boolean, got '${type}'`);
        return config[key];
    }
    getLogLevel(config, key, defaultValue) {
        if (typeof config[key] === "undefined")
            return defaultValue;
        const type = typeof config[key];
        if (type !== "string")
            throw new Error(`Got incorrect value type for config key '${key}' expected string, got '${type}'`);
        if (!Object.values(logging_1.NetworkDeviceLogLevel).includes(config[key]))
            throw new Error(`Got incorrect log level for config key '${key}': '${config[key]}'`);
        return logging_1.NetworkDeviceLogLevel[config[key]];
    }
}
exports.NetworkDeviceConfig = NetworkDeviceConfig;
//# sourceMappingURL=config.js.map