"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkDeviceLogger = exports.NetworkDeviceLogLevel = void 0;
var NetworkDeviceLogLevel;
(function (NetworkDeviceLogLevel) {
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Debug"] = 7] = "Debug";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Info"] = 6] = "Info";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Notice"] = 5] = "Notice";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Warning"] = 4] = "Warning";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Error"] = 3] = "Error";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Critical"] = 2] = "Critical";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Alert"] = 1] = "Alert";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["Emergency"] = 0] = "Emergency";
    NetworkDeviceLogLevel[NetworkDeviceLogLevel["None"] = -1] = "None";
})(NetworkDeviceLogLevel || (exports.NetworkDeviceLogLevel = NetworkDeviceLogLevel = {}));
class NetworkDeviceLogger {
    constructor(upstream, namespace) {
        this.level = NetworkDeviceLogLevel.Info;
        this.upstream = upstream;
        this.namespace = namespace;
    }
    debug(message, ...parameters) {
        // Use info instead of debug to control the level here instead of letting
        // homebridge take care of it - allows for it to be easily configured
        // per accessory
        if (this.level >= NetworkDeviceLogLevel.Debug)
            this.upstream.info(`[${this.namespace}] ${message}`, ...parameters);
    }
    info(message, ...parameters) {
        if (this.level >= NetworkDeviceLogLevel.Info)
            this.upstream.info(`[${this.namespace}] ${message}`, ...parameters);
    }
    warn(message, ...parameters) {
        if (this.level >= NetworkDeviceLogLevel.Warning)
            this.upstream.warn(`[${this.namespace}] ${message}`, ...parameters);
    }
    error(message, ...parameters) {
        if (this.level >= NetworkDeviceLogLevel.Error)
            this.upstream.error(`[${this.namespace}] ${message}`, ...parameters);
    }
}
exports.NetworkDeviceLogger = NetworkDeviceLogger;
//# sourceMappingURL=logging.js.map