"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const ping = __importStar(require("ping"));
const utilities_1 = require("./utilities");
const logging_1 = require("./logging");
class Pinger extends events_1.EventEmitter {
    constructor(upstreamLog, config) {
        super();
        this.timer = null;
        this.history = [];
        this.log = new logging_1.NetworkDeviceLogger(upstreamLog, "Pinger");
        this.config = config;
        this.log.level = this.config.logLevel;
    }
    /** Returns whether or not the target is considered alive. */
    async executePingCommand() {
        if (!this.config.pingCommand)
            throw new Error("Cannot execute ping command - no command configured");
        const commandName = this.config.pingCommand.split(" ")[0];
        this.log.debug('Attempting to poll state using "%s"', commandName);
        try {
            await (0, utilities_1.exec)(this.config.pingCommand, {
                timeout: this.config.pingCommandTimeout,
            });
            this.log.debug("Result of executing ping command: online");
            // If there is no error, the host is considered up
            return true;
        }
        catch (error) {
            this.log.debug("Result of executing ping command: offline");
            this.log.debug(error);
            // If there is an error, the host is considered down
            return false;
        }
    }
    /** Ping once. Returns whether or not the target is considered alive. */
    async ping() {
        if (!this.config.host)
            throw new Error("Cannot execute native ping - no host configured");
        this.log.debug("Attempting to ping host '%s'", this.config.host);
        // The timeout used by the ping library is in seconds
        const response = await ping.promise.probe(this.config.host, {
            timeout: this.config.pingTimeout / 1000,
        });
        this.log.debug("Result of pinging: %s", response.alive ? "online" : "offline");
        return response.alive;
    }
    /**
     * Poll the state of the target once.
     * @param immediate Whether or not to return the current state. Uses a buffer otherwise.
     */
    async pollState(immediate = false) {
        // Trust the ping command to return a true state
        if (this.config.pingCommand) {
            const isOnline = await this.executePingCommand();
            this.emit("stateChanged", isOnline);
            return;
        }
        // Use history for pinging, since it may be unreliable
        const isOnline = await this.ping();
        this.history.push(isOnline);
        // If there are not enough measurements yet, return prematurely
        if (!immediate && this.history.length < this.config.pingsToChange)
            return;
        const positives = this.history.reduce((positives, isPositive) => (isPositive ? positives + 1 : positives), 0);
        this.log.debug("Got %d positives out of %d", positives, this.history.length);
        // Only change state if all measurements are the same
        if (positives === this.history.length)
            this.emit("stateChanged", true);
        else if (positives === 0)
            this.emit("stateChanged", false);
        this.history = [];
    }
    /** Start polling the target's state. */
    async start() {
        this.stop();
        this.log.debug("Starting pinger at an interval of %d milliseconds", this.config.pingInterval);
        // Poll once immediately
        await this.pollState(true);
        // Start polling indefinitely
        this.timer = setInterval(() => {
            this.pollState();
        }, this.config.pingInterval);
    }
    /** Stop polling the target's state. */
    stop() {
        this.log.debug("Stopping pinger");
        if (this.timer)
            clearInterval(this.timer);
        this.history = [];
    }
}
exports.default = Pinger;
//# sourceMappingURL=pinger.js.map